<?php

namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\SM\Service\IntegracaoPmService;

class IntegracaoPmController extends SMCrud {
    
    protected $integracaoService;
    
    public function __construct($crudService, $environment, IntegracaoPmService $integracaoService) {
        parent::__construct($crudService, $environment);
        $this->integracaoService = $integracaoService;
    }
    
    /**
     * Insere uma proposta no SM a partir de um orçamento feito no PM
     * 
     * @param Request  $request
     * @param Response $response
     */
    public function salvarProposta(Request $request, Response $response) {
        
        try {
            $row = $request->getDataSet()->getRows()[0];
            $nrorg             = $row["requester"]["organizationId"];
            $cdoperador        = $row["requester"]["userId"];
            $codCliente        = $row["sponsor"];
            $codProduto        = $row["product"];
            $descricao         = $row["description"];
            $emailInterlocutor = $row["interlocutor"]["email"];
            $nomeInterlocutor  = $row["interlocutor"]["name"];
            $itens             = $row["items"];
            $desconto          = $row["discount"];
            $parcelas          = empty($row["instalments"]) ? null : $row["instalments"];
            $issueId           = empty($row["issueId"])     ? null : $row["issueId"];
            
            $codProposta = $this->integracaoService->prepararInserirProposta($nrorg, $cdoperador, $codCliente, $codProduto,
                $descricao, $emailInterlocutor, $nomeInterlocutor, $itens, $desconto, $parcelas, false, null, $issueId);

            $response->addDataSet(new DataSet("proposta", array("id" => $codProposta)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function aceitarProposta(Request $request, Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $nrorg          = $row["requester"]["organizationId"];
            $cdoperador     = $row["requester"]["userId"];
            $nrseqorcamento = $row["id"];

            $this->integracaoService->aceitarProposta($nrseqorcamento, $nrorg, $cdoperador);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function cancelarProposta(Request $request, Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $nrorg          = $row["requester"]["organizationId"];
            $cdoperador     = $row["requester"]["userId"];
            $nrseqorcamento = $row["id"];
            
            $this->integracaoService->cancelarProposta($nrseqorcamento, $nrorg, $cdoperador);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function gerarDocumentosProposta(Request $request, Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            
            $nrorg          = $row["requester"]["organizationId"];
            $cdoperador     = $row["requester"]["userId"];
            $nrseqorcamento = $row["id"];
            
            $caminhoArquivos = $this->integracaoService->gerarDocumentosProposta($nrseqorcamento, $nrorg);
            
            if (count($caminhoArquivos) > 0) {
                $links           = array();
                $caminhosFisicos = array();
                
                foreach ($caminhoArquivos as $chave => $arq)
                    $caminhosFisicos[$chave] = $arq["DADOSDOCUMENTO"]["DSCAMINHOARQ"];
                
                $retorno = $this->integracaoService->anexarDocumentosProposta($caminhosFisicos, $nrorg);
                
                if (count($retorno) > 0) {
                    $response->addDataSet(new DataSet("documentos", array("links" => $retorno)));
                    
                    foreach ($links as $arq) {
                        if (is_file($arq))
                            unlink($arq);
                    }
                } else {
                    throw new \Exception("Falha ao anexar o(s) documento(s) no FileServer.");
                }
            } else {
                throw new \Exception("Falha ao gerar o(s) documento(s).");
            }
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function aprovarContrato(Request $request, Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            
            $nrorg          = $row["requester"]["organizationId"];
            $cdoperador     = $row["requester"]["userId"];
            $nrseqorcamento = $row["id"];
            
            $this->integracaoService->aprovarContrato($nrseqorcamento, $nrorg, $cdoperador);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function salvarPropostaSolicitacoes() {
        $this->integracaoService->salvarPropostaSolicitacoes(20);
    }
    
    public function aprovarContratoSolicitacoes() {
        $this->integracaoService->aprovarContratoSolicitacoes(20);
    }
}

?>