<?php

namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\NewCode;
use Teknisa\SM\Service\Sm00008000HabilidadePessoalService;

class Sm00008000HabilidadePessoalController extends SMCrud {
    
    protected $dataSourceName = "fct_relachabilipessoa_widget";
    protected $newCode;
    protected $habilidadeService;
    
    public function __construct($crudService, $environment, NewCode $newCode, Sm00008000HabilidadePessoalService $habilidadeService) {
        parent::__construct($crudService, $environment);
        $this->newCode = $newCode;
        $this->habilidadeService = $habilidadeService;
    }
    
    public function save(Request $request, Response $response) {
        try {
            $nrorg = $this->environment->getNrOrg();
            
            foreach ($request->getDataSet()->getRows() as &$row) {
                $row["NRRELACHABILIPESSOA"] = $this->newCode->getCodeByTableName("FCT_RELACHABILIPESSOA", $nrorg, 1, 10, true);
            }
            unset($row);
            
            parent::save($request, $response);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    /**
     * Valida se já existem pessoas relacionadas à habilidade antes de inserí-las
     * 
     * @param Request $request
     * @param Response $response
     */
    public function validacao(Request $request, Response $response){
        try {
            $row = $request->getRow();
            $habilidade = $row["CDHABILITSERV"];
            $parceiros = $row["NRPARCNEGOCIO"];
            $nrorg = $this->environment->getNrOrg();
            
            $resultado = $this->habilidadeService->validacao($habilidade, $parceiros, $nrorg);
            
            $response->addDataSet(
                new DataSet(
                    "validacao", array(
                        "resultado" => $resultado
                    )
                )
            );
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
     
}

?>