<?php

namespace Teknisa\SM\Service;

use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Teknisa\SM\Service\Sm00001000LancamentoPropostaService;
use Teknisa\SM\Factory\Sm00013000GeracaoPropostaFactory;

class Sm00013000GeracaoPropostaService {
    
    protected $newCode;
    protected $connection;
    protected $propostaService;
    protected $geracaoPropostaFactory;
    
    public function __construct(NewCode $newCode, EntityManager $entityManager, Sm00001000LancamentoPropostaService $propostaService, Sm00013000GeracaoPropostaFactory $geracaoPropostaFactory) {
        $this->newCode                = $newCode;
        $this->connection             = $entityManager->getConnection();
        $this->propostaService        = $propostaService;
        $this->geracaoPropostaFactory = $geracaoPropostaFactory;
    }
    
    public function salvarEscoposGeracaoProposta($nrgeracaoproposta, $escopos, $cdoperador, $nrorg) {
        $params = array(
            "NRGERACAOPROPOSTA" => $nrgeracaoproposta,
            "NRORG"             => $nrorg
        );
        $this->geracaoPropostaFactory->excluirEscopos($params);
        
        $strEscopos = implode($escopos, ",");
        $this->geracaoPropostaFactory->excluirItensEscoposNaoSelecionados($params, $strEscopos);
        
        foreach ($escopos as $escopo) {
            $params = array(
                "NRGERACAOPROPOSTA" => $nrgeracaoproposta,
                "NRTIPOORCAMENTO"   => $escopo,
                "NRORG"             => $nrorg,
                "IDATIVO"           => "S",
                "NRORGINCLUSAO"     => $nrorg,
                "CDOPERINCLUSAO"    => $cdoperador
            );
            $this->geracaoPropostaFactory->inserirEscopos($params);
        }
    }
    
    public function inserirComposicoesElementosMedidas($nrgeracaoproposta, $elementos, $composicoes, $nrEscopos, $cdoperador, $nrorg) {
        $this->connection->beginTransaction();
        try {
            $params = array(
                "NRGERACAOPROPOSTA" => $nrgeracaoproposta,
                "NRORG"             => $nrorg
            );
            
            $this->geracaoPropostaFactory->excluirComposicoes($params);
            $this->geracaoPropostaFactory->excluirElementos($params);
            $this->geracaoPropostaFactory->excluirMedidas($params);
            
            foreach ($composicoes as $composicao) {
                $nrcompservgeracaoprop = $this->newCode->getCodeByTableName("FCT_COMPSERVGERACAOPROP", $nrorg, 1, 10, true);
                $params = array(
                    "NRCOMPSERVGERACAOPROP" => $nrcompservgeracaoprop,
                    "NRGERACAOPROPOSTA"     => $nrgeracaoproposta,
                    "NRCOMPSERVICO"         => $composicao["NRCOMPSERVICO"],
                    "NRCOMPSERVICOPAI"      => $composicao["NRCOMPSERVICOPAI"],
                    "IDCOMPOBRIGATORIA"     => $composicao["IDCOMPOBRIGATORIA"],
                    "IDESCOLHIDO"           => $composicao["IDESCOLHIDO"],
                    "NRORG"                 => $nrorg,
                    "IDATIVO"               => "S",
                    "NRORGINCLUSAO"         => $nrorg,
                    "CDOPERINCLUSAO"        => $cdoperador
                );
                
                $this->geracaoPropostaFactory->inserirComposicao($params);
            }
            
            $idElementosEscolhidos = array();
            foreach ($elementos as $elemento) {
                $nrelementogeracaoprop = $this->newCode->getCodeByTableName("FCT_ELEMENTOGERACAOPROP", $nrorg, 1, 10, true);
                $params = array(
                    "NRELEMENTOGERACAOPROP" => $nrelementogeracaoprop,
                    "NRGERACAOPROPOSTA"     => $nrgeracaoproposta,
                    "NRCOMPSERVICO"         => $elemento["NRCOMPSERVICO"],
                    "NRITCOMPSERVICO"       => $elemento["NRITCOMPSERVICO"],
                    "IDELEMENTO"            => $elemento["IDELEMENTO"],
                    "IDELEMENTOPAI"         => $elemento["IDELEMENTOPAI"],
                    "IDTIPOELEMENTO"        => $elemento["IDTIPOELEMENTO"],
                    "IDSERVOBRIGCOMP"       => $elemento["IDSERVOBRIGCOMP"],
                    "IDESCOLHIDO"           => $elemento["IDESCOLHIDO"],
                    "NRORG"                 => $nrorg,
                    "IDATIVO"               => "S",
                    "NRORGINCLUSAO"         => $nrorg,
                    "CDOPERINCLUSAO"        => $cdoperador
                );
                
                $this->geracaoPropostaFactory->inserirElemento($params);
                
                if ($elemento["IDESCOLHIDO"] == "S") {
                    $idElementosEscolhidos[] = $elemento["IDELEMENTO"];
                }
            }
            
            $medidas = $this->propostaService->getTiposMedidaVariaveisDosElementos(null, $idElementosEscolhidos, $nrEscopos, $nrorg);
            
            foreach ($medidas as $medida) {
                //$nrmedvargeracaoprop = $this->newCode->getCodeByTableName("FCT_MEDVARGERACAOPROP", $nrorg, 1, 10, true);
                $params = array(
                    //"NRMEDVARGERACAOPROP" => $nrmedvargeracaoprop,
                    "NRGERACAOPROPOSTA"   => $nrgeracaoproposta,
                    "IDELEMENTORAIZ"      => $medida["IDELEMENTORAIZ"],
                    "CDTPMEDIDAVARIAVEL"  => $medida["CDTPMEDIDASERV"],
                    "QTMINIMA"            => $medida["QUANTIDADE"],
                    "QTMAXIMA"            => 99999999,
                    "QTFRANQUIA"          => $medida["QTFRANQUIA"],
                    "NRORG"               => $nrorg,
                    "IDATIVO"             => "S",
                    "NRORGINCLUSAO"       => $nrorg,
                    "CDOPERINCLUSAO"      => $cdoperador
                );
                
                $this->geracaoPropostaFactory->inserirMedidas($params);
            }
            
            $this->connection->commit();
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function gerarJsonIntegracao($nrgeracaoproposta, $nrorg) {
        $arrayCliente = $this->getArrayIntegracaoParteDeCliente();
        $arrayInterlocutor = $this->getArrayIntegracaoParteDeInterlocutor();
        $arrayProposta = $this->getArrayIntegracaoParteDeProposta($nrgeracaoproposta, $nrorg);
        
        $jsonIntegracao = array(
            "cliente"      => $arrayCliente,
            "interlocutor" => $arrayInterlocutor,
            "proposta"     => $arrayProposta
        );
        
        return $jsonIntegracao;
    }
    
    private function getArrayIntegracaoParteDeCliente() {
        return array(
            "razao_social"        => "<STRING>",
            "nome_fantasia"       => "<STRING>",
            "cpf_cnpj"            => "<STRING>",
            "inscricao_estadual"  => "<STRING>",
            "inscricao_municipal" => "<STRING>",
            "endereco"            => "<STRING>",
            "bairro"              => "<STRING>",
            "cidade"              => "<STRING>",
            "estado"              => "<STRING>",
            "telefone"            => "<STRING>"
        );
    }
    
    private function getArrayIntegracaoParteDeInterlocutor() {
        return array(
            "nome"     => "<STRING>",
            "email"    => "<STRING>",
            "cpf"      => "<STRING>",
            "telefone" => "<STRING>"
        );
    }
    
    private function getArrayIntegracaoParteDeProposta($nrgeracaoproposta, $nrorg) {
        $arrayMedidas = array();
        
        $params = array(
            "NRGERACAOPROPOSTA" => $nrgeracaoproposta,
            "NRORG" => $nrorg
        );
        $medidasVariaveis = $this->geracaoPropostaFactory->getMedidasVariaveis($params);
        foreach ($medidasVariaveis as $medida) {
            $arrayMedidas[] = array(
                "id_elemento"   => $medida["IDELEMENTORAIZ"],
                "nome_elemento" => $medida["NMELEMENTORAIZ"],
                "id_medida"     => $medida["CDTPMEDIDAVARIAVEL"],
                "nome_medida"   => $medida["DSTPMEDIDAVARIAVEL"],
                "qtd_medida"    => "<FLOAT>"
            );
        }
        
        $arrayParcelamento = array();
        $escopos = $this->geracaoPropostaFactory->getEscopos($params);
        foreach ($escopos as $escopo) {
            $arrayParcelamento[] = array(
                "id_escopo"    => (int)$escopo["NRTIPOORCAMENTO"],
                "nome_escopo"  => $escopo["NMTIPOORCAMENTO"],
                "qtd_parcelas" => "<INT>"
            );
        }
        
        $arrayProposta = array(
            "id_layout"         => (int)$nrgeracaoproposta,
            "medidas_variaveis" => $arrayMedidas,
            "parcelamento"      => $arrayParcelamento
        );
        
        return $arrayProposta;
    }
}