/*
 @C:\Web_565\Sql\End_Conv\SQLServer\1065_FINSTPS_GERA_AJUSTE_NFDEVOLUCAO.SQL

 Descrio:
 Procedure para que ao aprovar uma nota de devoluo seje gerado desconto no boleto

 Parmetros

 @V_CDEMPRESA    - Cdigo da empresa
 @V_IDENTRSAIDOP - Verifica se a nota fiscal  de entrada ou sada
 @V_RETORNO      - Retorno da procedure CANCELA_DADOS_FINANCEIRO
 @V_VRDISPONIVEL - Valor disponvel da nota fiscal de devoluo
 @V_VRAJUSTE     - Valor do ajuste
 @V_CDAJUCTPAG   - Cdigo do tipo de ajuste de conta a Pagar
 @V_CDAJUCTREC   - Cdigo do tipo de ajuste de conta a Receber
 @V_VRDEVOLUTI   - Valor devolvido
 @V_EXISTEAJUS   - Verifica se existe ajuste no boleto
 @V_DEVOLUCAOREC - Gera um cdigo sequncial na devoluo
 @V_DEVOLUCAOPAG - Gera um cdigo sequncial na devoluo

 RAISERROR('', 16,1);
*/

IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'GERA_AJUSTE_NFDEVOLUCAO' AND
                 TYPE = 'P' 		             AND
	              U.UID = O.UID                  AND
	              U.NAME = USER)
DROP PROCEDURE GERA_AJUSTE_NFDEVOLUCAO
GO
CREATE PROCEDURE GERA_AJUSTE_NFDEVOLUCAO (@P_CDFILIAL   VARCHAR(04),
                                          @P_NRLANCTONF VARCHAR(10))
AS
   DECLARE @V_CDEMPRESA    VARCHAR(02);
   DECLARE @V_IDENTRSAIDOP VARCHAR(01);
   DECLARE @V_RETORNO      NUMERIC(1);
   DECLARE @V_VRDISPONIVEL FLOAT;
   DECLARE @V_VRAJUSTE     FLOAT;
   DECLARE @V_CDAJUCTPAG   VARCHAR(04);
   DECLARE @V_CDAJUCTREC   VARCHAR(04);
   DECLARE @V_VRDEVOLUTI   FLOAT;
   DECLARE @V_EXISTEAJUS   NUMERIC(2);

   /*VARIVEIS DO CURSOR C_NOTAS*/
   DECLARE @V_NOTAS_CDFILIALIT   VARCHAR(04);
   DECLARE @V_NOTAS_NRLANCTONFIT VARCHAR(10);

   /*VARIVEIS DO CURSOR C_CONTAS*/
   DECLARE @V_CONTAS_CDFILIAL VARCHAR(04);
   DECLARE @V_CONTAS_NRLANBOLPAG VARCHAR(10);
   DECLARE @V_CONTAS_VRFINALBPAG FLOAT;
   DECLARE @V_CONTAS_NRLANBOLREC VARCHAR(10);
   DECLARE @V_CONTAS_VRFINALBREC FLOAT;

   /*VARIAVEL DE DEVOLUCAO*/
   DECLARE @V_DEVOLUCAOPAG VARCHAR(20);
   DECLARE @V_DEVOLUCAOREC VARCHAR(20);
   DECLARE @V_CHAVE VARCHAR(20);

   BEGIN

   SET @V_VRDEVOLUTI = 0;
   
   SELECT @V_IDENTRSAIDOP = NF.IDENTRSAIDOP,
          @V_CDEMPRESA    = NF.CDEMPRESA,
          @V_VRDISPONIVEL = ROUND((NF.VRNOTAFISC - ISNULL(NF.VRDEVOLNFUTL, 0)), 2)
     FROM NOTAFISCAL NF
    WHERE NF.CDFILIAL      = @P_CDFILIAL
      AND NF.NRLANCTONF    = @P_NRLANCTONF;

   /* Verifica se est parametrizado o Tipo de Ajuste */
   SELECT @V_CDAJUCTPAG = MAX(CDAJUCTPAG),
          @V_CDAJUCTREC = MAX(CDAJUCTREC)
     FROM PARAMEMPRESA
    WHERE CDEMPRESA = @V_CDEMPRESA;

   IF @V_IDENTRSAIDOP = 'S' BEGIN
      IF NOT @V_CDAJUCTPAG IS NULL BEGIN
         /* Verifca a nota fiscal de origem */
         DECLARE C_NOTAS CURSOR FOR SELECT CDFILIALIT, NRLANCTONFIT
                                      FROM ITEMNOTAFISC
                                     WHERE (CDFILIAL   = @P_CDFILIAL)
                                       AND (NRLANCTONF = @P_NRLANCTONF);
         OPEN C_NOTAS;
         FETCH NEXT FROM C_NOTAS INTO @V_NOTAS_CDFILIALIT, @V_NOTAS_NRLANCTONFIT;

         WHILE (@@FETCH_STATUS = 0) BEGIN

            EXEC CANCELA_DADOS_FINANCEIRO @V_NOTAS_CDFILIALIT,
                                          @V_NOTAS_NRLANCTONFIT,
                                          'N',
                                          @V_RETORNO OUTPUT;

            /* RETORNO = 0 - CANCELAMENTO FINANCEIRO CONCLUDO
               RETORNO = 4 - BOLETO COM AJUSTE */
            IF (@V_RETORNO = 0 OR @V_RETORNO = 4) BEGIN
               DECLARE C_CONTAS CURSOR FOR SELECT BO.CDFILIAL, BO.NRLANBOLPAG, ROUND(BO.VRFINALBPAG, 2) AS VRFINALBPAG
                                             FROM CONTAPAGAR CP, BOLETPAGAR BO
                                            WHERE CP.CDFILIAL    = @V_NOTAS_CDFILIALIT
                                              AND CP.NRLANCTONF  = @V_NOTAS_NRLANCTONFIT
                                              AND CP.CDFILIBPAG  = BO.CDFILIAL
                                              AND CP.NRLANBOLPAG = BO.NRLANBOLPAG;
               OPEN C_CONTAS;
               FETCH NEXT FROM C_CONTAS INTO @V_CONTAS_CDFILIAL, @V_CONTAS_NRLANBOLPAG, @V_CONTAS_VRFINALBPAG;

               WHILE (@@FETCH_STATUS = 0) BEGIN

                  SET @V_VRAJUSTE = 0;

                  IF @V_VRDISPONIVEL > 0 BEGIN
                     IF @V_CONTAS_VRFINALBPAG > @V_VRDISPONIVEL BEGIN
                        SET @V_VRAJUSTE = ROUND(@V_VRDISPONIVEL, 2);
                     END
                     ELSE BEGIN
                        SET @V_VRAJUSTE = ROUND(@V_CONTAS_VRFINALBPAG, 2);
                     END;

                     SET @V_VRDISPONIVEL = ROUND((@V_VRDISPONIVEL - @V_VRAJUSTE), 2);
                     /* Verifica se existe ajuste */
                     SELECT @V_EXISTEAJUS = COUNT(CDAJUCTPAG)
                       FROM AJBOLPAG
                      WHERE CDFILIAL    = @V_CONTAS_CDFILIAL
                        AND NRLANBOLPAG = @V_CONTAS_NRLANBOLPAG
                        AND CDAJUCTPAG  = @V_CDAJUCTPAG;

                     IF @V_EXISTEAJUS > 0 BEGIN
                        UPDATE AJBOLPAG
                           SET VRAJBOLPAG  = ROUND(VRAJBOLPAG + @V_VRAJUSTE, 2)
                         WHERE CDFILIAL    = @V_CONTAS_CDFILIAL
                           AND NRLANBOLPAG = @V_CONTAS_NRLANBOLPAG
                           AND CDAJUCTPAG  = @V_CDAJUCTPAG;
                     END
                     ELSE BEGIN
                        INSERT INTO AJBOLPAG
                           (CDAJUCTPAG, VRAJBOLPAG, CDFILIAL, NRLANBOLPAG)
                        VALUES
                           (@V_CDAJUCTPAG, @V_VRAJUSTE, @V_CONTAS_CDFILIAL, @V_CONTAS_NRLANBOLPAG);
                     END;

                     SET @V_CHAVE = 'BOLETPAGAR';
                     EXEC NOVO_CODIGO @V_CHAVE, @V_DEVOLUCAOPAG OUTPUT, 1 ;
                     SET @V_DEVOLUCAOPAG = SUBSTRING(@V_DEVOLUCAOPAG, LEN(@V_DEVOLUCAOPAG) - 9, 10);

                     INSERT INTO DEVOLUCAOPAG
                        (NRDEVOLUCAOPAG, CDFILIBOLPAG, NRLANBOLPAG, CDAJUCTPAG, CDFILIAL, NRLANCTONF, VRDEVOLUCAOPAG)
                     VALUES
                       (@V_DEVOLUCAOPAG, @V_CONTAS_CDFILIAL, @V_CONTAS_NRLANBOLPAG, @V_CDAJUCTPAG, @P_CDFILIAL, @P_NRLANCTONF, ROUND(@V_VRAJUSTE, 2));

                     UPDATE BOLETPAGAR
                        SET VRFINALBPAG = ROUND(VRFINALBPAG - @V_VRAJUSTE, 2)
                      WHERE CDFILIAL    = @V_CONTAS_CDFILIAL
                        AND NRLANBOLPAG = @V_CONTAS_NRLANBOLPAG;

                     SET @V_VRDEVOLUTI = ROUND(ISNULL(@V_VRDEVOLUTI, 0) + ISNULL(@V_VRAJUSTE, 0), 2);
                   END;

                  FETCH NEXT FROM C_CONTAS INTO @V_CONTAS_CDFILIAL, @V_CONTAS_NRLANBOLPAG, @V_CONTAS_VRFINALBPAG;
               END;
               CLOSE C_CONTAS;
               DEALLOCATE C_CONTAS;
            END;
            
            FETCH NEXT FROM C_NOTAS INTO @V_NOTAS_CDFILIALIT, @V_NOTAS_NRLANCTONFIT;
         END;
         CLOSE C_NOTAS;
         DEALLOCATE C_NOTAS;
      END;
   END
   ELSE IF @V_IDENTRSAIDOP = 'E' BEGIN
      IF @V_IDENTRSAIDOP = 'E' BEGIN
         IF NOT @V_CDAJUCTREC IS NULL BEGIN
            /* Verifca a nota fiscal de origem */
            DECLARE C_NOTAS CURSOR FOR SELECT CDFILIALIT, NRLANCTONFIT
                                         FROM ITEMNOTAFISC
                                        WHERE (CDFILIAL   = @P_CDFILIAL)
                                          AND (NRLANCTONF = @P_NRLANCTONF);
            OPEN C_NOTAS;
            FETCH NEXT FROM C_NOTAS INTO @V_NOTAS_CDFILIALIT, @V_NOTAS_NRLANCTONFIT;

            WHILE (@@FETCH_STATUS = 0) BEGIN

               EXEC CANCELA_DADOS_FINANCEIRO @V_NOTAS_CDFILIALIT,
                                             @V_NOTAS_NRLANCTONFIT,
                                             'N',
                                             @V_RETORNO OUTPUT;

               /* RETORNO = 0 - CANCELAMENTO FINANCEIRO CONCLUDO
                  RETORNO = 4 - BOLETO COM AJUSTE */
               IF (@V_RETORNO = 0 OR @V_RETORNO = 4) BEGIN
                  DECLARE C_CONTAS CURSOR FOR SELECT BO.CDFILIAL, BO.NRLANBOLREC, ROUND(BO.VRFINALBREC, 2) AS VRFINALBREC
                                                FROM CONTARECEBER CR, BOLETRECEBER BO
                                               WHERE CR.CDFILIAL    = @V_NOTAS_CDFILIALIT
                                                 AND CR.NRLANCTONF  = @V_NOTAS_NRLANCTONFIT
                                                 AND CR.CDFILIBOLREC  = BO.CDFILIAL
                                                 AND CR.NRLANBOLREC = BO.NRLANBOLREC;
                  OPEN C_CONTAS;
                  FETCH NEXT FROM C_CONTAS INTO @V_CONTAS_CDFILIAL, @V_CONTAS_NRLANBOLREC, @V_CONTAS_VRFINALBREC;

                  WHILE (@@FETCH_STATUS = 0) BEGIN

                     SET @V_VRAJUSTE = 0;

                     IF @V_VRDISPONIVEL > 0 BEGIN
                        IF @V_CONTAS_VRFINALBREC > @V_VRDISPONIVEL BEGIN
                           SET @V_VRAJUSTE = ROUND(@V_VRDISPONIVEL, 2);
                        END
                        ELSE BEGIN
                           SET @V_VRAJUSTE = ROUND(@V_CONTAS_VRFINALBREC, 2);
                        END;

                        SET @V_VRDISPONIVEL = ROUND(@V_VRDISPONIVEL - @V_VRAJUSTE, 2);
                        /* Verifica se existe ajuste */
                        SELECT @V_EXISTEAJUS = COUNT(CDAJUCTREC)
                          FROM AJBOLREC
                         WHERE CDFILIAL    = @V_CONTAS_CDFILIAL
                           AND NRLANBOLREC = @V_CONTAS_NRLANBOLREC
                           AND CDAJUCTREC  = @V_CDAJUCTREC;

                        IF @V_EXISTEAJUS > 0 BEGIN
                           UPDATE AJBOLREC
                              SET VRAJBOLREC  = ROUND(VRAJBOLREC + @V_VRAJUSTE, 2)
                            WHERE CDFILIAL    = @V_CONTAS_CDFILIAL
                              AND NRLANBOLREC = @V_CONTAS_NRLANBOLREC
                              AND CDAJUCTREC  = @V_CDAJUCTREC;
                        END
                        ELSE BEGIN
                           INSERT INTO AJBOLREC
                              (CDAJUCTREC, VRAJBOLREC, CDFILIAL, NRLANBOLREC)
                           VALUES
                              (@V_CDAJUCTREC, @V_VRAJUSTE, @V_CONTAS_CDFILIAL, @V_CONTAS_NRLANBOLREC);
                        END;

                        SET @V_CHAVE = 'BOLETRECEBER';
                        EXEC NOVO_CODIGO @V_CHAVE, @V_DEVOLUCAOREC OUTPUT, 1 ;
                        SET @V_DEVOLUCAOREC = SUBSTRING(@V_DEVOLUCAOREC, LEN(@V_DEVOLUCAOREC) - 9, 10);

                        INSERT INTO DEVOLUCAOREC
                           (NRDEVOLUCAOREC, CDFILIBOLREC, NRLANBOLREC, CDAJUCTREC, CDFILIAL, NRLANCTONF, VRDEVOLUCAOREC)
                        VALUES
                          (@V_DEVOLUCAOREC, @V_CONTAS_CDFILIAL, @V_CONTAS_NRLANBOLREC, @V_CDAJUCTREC, @P_CDFILIAL, @P_NRLANCTONF, ROUND(@V_VRAJUSTE, 2));

                        UPDATE BOLETRECEBER
                           SET VRFINALBREC = ROUND(VRFINALBREC - @V_VRAJUSTE, 2)
                         WHERE CDFILIAL    = @V_CONTAS_CDFILIAL
                           AND NRLANBOLREC = @V_CONTAS_NRLANBOLREC;

                        SET @V_VRDEVOLUTI = ROUND(ISNULL(@V_VRDEVOLUTI, 0) + ISNULL(@V_VRAJUSTE, 0), 2);
                     END;

                     FETCH NEXT FROM C_CONTAS INTO @V_CONTAS_CDFILIAL, @V_CONTAS_NRLANBOLREC, @V_CONTAS_VRFINALBREC;
                  END;
                  CLOSE C_CONTAS;
                  DEALLOCATE C_CONTAS;
               END;

               FETCH NEXT FROM C_NOTAS INTO @V_NOTAS_CDFILIALIT, @V_NOTAS_NRLANCTONFIT;
            END;
            CLOSE C_NOTAS;
            DEALLOCATE C_NOTAS;
         END;
      END;
   END;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'GERA_AJUSTE_NFDEVOLUCAO','5.65.xxx';
GO
